<?php
// +----------------------------------------------------------------------
// | Leaps Framework [ WE CAN DO IT JUST THINK IT ]
// +----------------------------------------------------------------------
// | Copyright (c) 2006-2014 http://www.tintsoft.com All rights reserved.
// +----------------------------------------------------------------------
// | Licensed ( http://www.apache.org/licenses/LICENSE-2.0 )
// +----------------------------------------------------------------------
// | Author XuTongle <xutongle@gmail.com>
// +----------------------------------------------------------------------
// | $Id$
// +----------------------------------------------------------------------
class Request {

	public function __construct(){
		//$this->init();
	}
	/**
	 * 初始化request对象
	 *
	 * @see \Leaps\Base\Object::init()
	 */
	public function init() {
		if (function_exists ( 'get_magic_quotes_gpc' ) && get_magic_quotes_gpc ()) {
			if (isset ( $_GET ))
				$_GET = $this->_stripSlashes ( $_GET );
			if (isset ( $_POST ))
				$_POST = $this->_stripSlashes ( $_POST );
			if (isset ( $_REQUEST ))
				$_REQUEST = $this->_stripSlashes ( $_REQUEST );
			if (isset ( $_COOKIE ))
				$_COOKIE = $this->_stripSlashes ( $_COOKIE );
		}
	}


	private $_headers;

	/**
	 * 返回Header集合。
	 *
	 * @return HeaderCollection the header collection
	 */
	public function getHeaders() {
		if ($this->_headers === null) {
			if (function_exists ( 'getallheaders' )) {
				$headers = getallheaders ();
			} elseif (function_exists ( 'http_get_request_headers' )) {
				$headers = http_get_request_headers ();
			} elseif (function_exists ( 'apache_request_headers' )) {
				$headers = apache_request_headers ();
			} else {
				foreach ( $_SERVER as $name => $value ) {
					if (strncmp ( $name, 'HTTP_', 5 ) === 0) {
						$name = str_replace ( ' ', '-', ucwords ( strtolower ( str_replace ( '_', ' ', substr ( $name, 5 ) ) ) ) );
						$this->_headers [$name] = $value;
					}
				}
				return $this->_headers;
			}
			foreach ( $headers as $name => $value ) {
				$this->_headers [$name] = $value;
			}
		}
		return $this->_headers;
	}
	public $methodParam = '_method';

	/**
	 * 返回当前请求的方法 (e.g.
	 * GET, POST, HEAD, PUT, PATCH, DELETE).
	 *
	 * @return string 请求方法, 比如 GET, POST, HEAD, PUT, PATCH, DELETE.
	 */
	public function getMethod() {
		if (isset ( $_POST [$this->methodParam] )) {
			return strtoupper ( $_POST [$this->methodParam] );
		} elseif (isset ( $_SERVER ['HTTP_X_HTTP_METHOD_OVERRIDE'] )) {
			return strtoupper ( $_SERVER ['HTTP_X_HTTP_METHOD_OVERRIDE'] );
		} else {
			return isset ( $_SERVER ['REQUEST_METHOD'] ) ? strtoupper ( $_SERVER ['REQUEST_METHOD'] ) : 'GET';
		}
	}

	/**
	 * 返回Server的值 如果$name为空则返回所有Server的值
	 *
	 * @param string $name 获取的变量名,如果该值为null则返回$_SERVER数组,默认为null
	 * @param string $defaultValue 当获取变量失败的时候返回该值,默认该值为null
	 * @return mixed
	 */
	public function getServer($name = null, $defaultValue = null) {
		if ($name === null)
			return $_SERVER;
		return (isset ( $_SERVER [$name] )) ? $_SERVER [$name] : $defaultValue;
	}

	/**
	 * 返回ENV的值 如果$name为null则返回所有$_ENV的值
	 *
	 * @param string $name 获取的变量名,如果该值为null则返回$_ENV数组,默认为null
	 * @param string $defaultValue 当获取变量失败的时候返回该值,默认该值为null
	 * @return mixed
	 */
	public function getEnv($name = null, $defaultValue = null) {
		if ($name === null)
			return $_ENV;
		return (isset ( $_ENV [$name] )) ? $_ENV [$name] : $defaultValue;
	}
	private $_rawBody;

	/**
	 * 返回原始HTTP请求体。
	 *
	 * @return string the request body
	 */
	public function getRawBody() {
		if ($this->_rawBody === null) {
			$this->_rawBody = file_get_contents ( 'php://input' );
		}
		return $this->_rawBody;
	}

	/**
	 * 返回是否是GET请求。
	 *
	 * @return boolean whether this is a GET request.
	 */
	public function isGet() {
		return $this->getMethod () === 'GET';
	}

	/**
	 * 返回是否是Options请求。
	 *
	 * @return boolean whether this is a OPTIONS request.
	 */
	public function isOptions() {
		return $this->getMethod () === 'OPTIONS';
	}

	/**
	 * 返回是否是Head请求。
	 *
	 * @return boolean whether this is a HEAD request.
	 */
	public function isHead() {
		return $this->getMethod () === 'HEAD';
	}

	/**
	 * 返回是否是Post请求。
	 *
	 * @return boolean whether this is a POST request.
	 */
	public function isPost() {
		return $this->getMethod () === 'POST';
	}

	/**
	 * 返回是否是Delete请求。
	 *
	 * @return boolean whether this is a DELETE request.
	 */
	public function isDelete() {
		return $this->getMethod () === 'DELETE';
	}

	/**
	 * 返回是否是Put请求。
	 *
	 * @return boolean whether this is a PUT request.
	 */
	public function isPut() {
		return $this->getMethod () === 'PUT';
	}

	/**
	 * 返回是否是PATCH请求。
	 *
	 * @return boolean whether this is a PATCH request.
	 */
	public function isPatch() {
		return $this->getMethod () === 'PATCH';
	}

	/**
	 * 返回是否是AJAX (XMLHttpRequest)请求。
	 *
	 * @return boolean whether this is an AJAX (XMLHttpRequest) request.
	 */
	public function isAjax() {
		return isset ( $_SERVER ['HTTP_X_REQUESTED_WITH'] ) && $_SERVER ['HTTP_X_REQUESTED_WITH'] === 'XMLHttpRequest';
	}

	/**
	 * 返回是否是PJAX请求。
	 *
	 * @return boolean whether this is a PJAX request
	 */
	public function isPjax() {
		return $this->getIsAjax () && ! empty ( $_SERVER ['HTTP_X_PJAX'] );
	}

	/**
	 * 返回是否是 Adobe Flash 或 Flex 请求。
	 *
	 * @return boolean whether this is an Adobe Flash or Adobe Flex request.
	 */
	public function isFlash() {
		return isset ( $_SERVER ['HTTP_USER_AGENT'] ) && (stripos ( $_SERVER ['HTTP_USER_AGENT'], 'Shockwave' ) !== false || stripos ( $_SERVER ['HTTP_USER_AGENT'], 'Flash' ) !== false);
	}

	/**
	 * 返回是否是手机浏览器发起的请求
	 *
	 * @return boolean
	 */
	public function isMobile() {
		$regex_match = "/(nokia|iphone|android|motorola|^mot\-|softbank|foma|docomo|kddi|up\.browser|up\.link|";
		$regex_match .= "htc|dopod|blazer|netfront|helio|hosin|huawei|novarra|CoolPad|webos|techfaith|palmsource|";
		$regex_match .= "blackberry|alcatel|amoi|ktouch|nexian|samsung|^sam\-|s[cg]h|^lge|ericsson|philips|sagem|wellcom|bunjalloo|maui|";
		$regex_match .= "symbian|smartphone|midp|wap|phone|windows ce|iemobile|^spice|^bird|^zte\-|longcos|pantech|gionee|^sie\-|portalmmm|";
		$regex_match .= "jig\s browser|hiptop|^ucweb|^benq|haier|^lct|opera\s*mobi|opera\*mini|320x320|240x320|176x220";
		$regex_match .= ")/i";
		return isset ( $_SERVER ['HTTP_X_WAP_PROFILE'] ) or isset ( $_SERVER ['HTTP_PROFILE'] ) or preg_match ( $regex_match, strtolower ( $_SERVER ['HTTP_USER_AGENT'] ) );
	}
	private $_bodyParams;
	public $parsers = array();

	/**
	 * 返回请求主体的参数。
	 *
	 * @return array the request parameters given in the request body.
	 * @throws \Leaps\Base\InvalidConfigException if a registered parser does
	 *         not implement the
	 *         [[RequestParserInterface]].
	 * @see getMethod()
	 * @see getBodyParam()
	 * @see setBodyParams()
	 */
	public function getBodyParams() {
		if ($this->_bodyParams === null) {
			$contentType = $this->getContentType ();
			if (isset ( $_POST [$this->methodParam] )) {
				$this->_bodyParams = $_POST;
				unset ( $this->_bodyParams [$this->methodParam] );
			} elseif (isset ( $this->parsers [$contentType] )) {
				$parser = createObject ( $this->parsers [$contentType] );
				//if (! ($parser instanceof \Leaps\RequestParserInterface)) {
				//	throw new InvalidConfigException ( "The '$contentType' request parser is invalid. It must implement the yii\\web\\RequestParserInterface." );
				//}
				$this->_bodyParams = $parser->parse ( $this->getRawBody (), $contentType );
			} elseif (isset ( $this->parsers ['*'] )) {
				$parser = createObject ( $this->parsers ['*'] );
				//if (! ($parser instanceof \Leaps\RequestParserInterface)) {
				//	throw new InvalidConfigException ( "The fallback request parser is invalid. It must implement the yii\\web\\RequestParserInterface." );
				//}
				$this->_bodyParams = $parser->parse ( $this->getRawBody (), $contentType );
			} elseif ($this->getMethod () === 'POST') {
				// PHP has already parsed the body so we have all params in
				// $_POST
				$this->_bodyParams = $_POST;
			} else {
				$this->_bodyParams = [ ];
				mb_parse_str ( $this->getRawBody (), $this->_bodyParams );
			}
		}
		return $this->_bodyParams;
	}

	/**
	 * 设置请求体参数。
	 *
	 * @param array $values the request body parameters (name-value pairs)
	 * @see getBodyParam()
	 * @see getBodyParams()
	 */
	public function setBodyParams($values) {
		$this->_bodyParams = $values;
	}

	/**
	 * 返回指定请求参数值。
	 *
	 * @param string $name the parameter name
	 * @param mixed $defaultValue the default parameter value if the parameter
	 *        does not exist.
	 * @return mixed the parameter value
	 * @see getBodyParams()
	 * @see setBodyParams()
	 */
	public function getBodyParam($name, $defaultValue = null) {
		$params = $this->getBodyParams ();
		return isset ( $params [$name] ) ? $params [$name] : $defaultValue;
	}

	/**
	 * 返回Post参数
	 *
	 * @param string $name the parameter name
	 * @param mixed $defaultValue the default parameter value if the parameter
	 *        does not exist.
	 * @return array mixed
	 */
	public function post($name = null, $defaultValue = null) {
		if ($name === null) {
			return $this->getBodyParams ();
		} else {
			return $this->getBodyParam ( $name, $defaultValue );
		}
	}
	private $_queryParams;

	/**
	 * 返回[[queryString]]中给出的请求参数 This method will return the contents of `$_GET`
	 * if params where not
	 * explicitly set.
	 *
	 * @return array the request GET parameter values.
	 * @see setQueryParams()
	 */
	public function getQueryParams() {
		if ($this->_queryParams === null) {
			return $_GET;
		}
		return $this->_queryParams;
	}

	/**
	 * 设置[[queryString]]中的参数
	 *
	 * @param array $values the request query parameters (name-value pairs)
	 * @see getQueryParam()
	 * @see getQueryParams()
	 */
	public function setQueryParams($values) {
		$this->_queryParams = $values;
	}

	/**
	 * 返回Get参数
	 *
	 * @param string $name the parameter name
	 * @param mixed $defaultValue the default parameter value if the parameter
	 *        does not exist.
	 * @return array mixed
	 */
	public function get($name = null, $defaultValue = null) {
		if ($name === null) {
			return $this->getQueryParams ();
		} else {
			return $this->getQueryParam ( $name, $defaultValue );
		}
	}

	/**
	 * 返回指定参数值。 不存在将返回第二个参数
	 *
	 * @param string $name the GET parameter name. If not specified, whole $_GET
	 *        is
	 *        returned.
	 * @param mixed $defaultValue the default parameter value if the GET
	 *        parameter does not
	 *        exist.
	 * @return mixed the GET parameter value
	 * @see getBodyParam()
	 */
	public function getQueryParam($name, $defaultValue = null) {
		$params = $this->getQueryParams ();
		return isset ( $params [$name] ) ? $params [$name] : $defaultValue;
	}
	private $_hostInfo;

	/**
	 * 获得主机信息，包含协议信息，主机名，访问端口信息
	 *
	 * @return string schema and hostname part (with port number if needed) of
	 *         the request URL (e.g.
	 *         `http://www.tintsoft.com`)
	 * @see setHostInfo()
	 */
	public function getHostInfo() {
		if ($this->_hostInfo === null) {
			$secure = $this->isSecureConnection ();
			$http = $secure ? 'https' : 'http';
			if (isset ( $_SERVER ['HTTP_HOST'] )) {
				$this->_hostInfo = $http . '://' . $_SERVER ['HTTP_HOST'];
			} else {
				$this->_hostInfo = $http . '://' . $_SERVER ['SERVER_NAME'];
				$port = $secure ? $this->getSecurePort () : $this->getPort ();
				if (($port !== 80 && ! $secure) || ($port !== 443 && $secure)) {
					$this->_hostInfo .= ':' . $port;
				}
			}
		}

		return $this->_hostInfo;
	}

	/**
	 * 设置主机信息，包含协议信息，主机名，访问端口信息
	 *
	 * @param string $value the schema and host part of the application URL. The
	 *        trailing
	 *        slashes
	 *        will be removed.
	 */
	public function setHostInfo($value) {
		$this->_hostInfo = rtrim ( $value, '/' );
	}
	private $_baseUrl;

	/**
	 * 返回应用程序的相对URL。
	 *
	 * @return string the relative URL for the application
	 * @see setScriptUrl()
	 */
	public function getBaseUrl() {
		if ($this->_baseUrl === null) {
			$this->_baseUrl = rtrim ( dirname ( $this->getScriptUrl () ), '\\/' );
		}
		return $this->_baseUrl;
	}

	/**
	 * 设置应用程序的相对URL。
	 *
	 * @param string $value the relative URL for the application
	 */
	public function setBaseUrl($value) {
		$this->_baseUrl = $value;
	}
	private $_scriptUrl;

	/**
	 * 返回当前执行脚本的相对路径
	 *
	 * @return string the relative URL of the entry script.
	 * @throws InvalidConfigException if unable to determine the entry script
	 *         URL
	 */
	public function getScriptUrl() {
		if ($this->_scriptUrl === null) {
			$scriptFile = $this->getScriptFile ();
			$scriptName = basename ( $scriptFile );
			if (basename ( $_SERVER ['SCRIPT_NAME'] ) === $scriptName) {
				$this->_scriptUrl = $_SERVER ['SCRIPT_NAME'];
			} elseif (basename ( $_SERVER ['PHP_SELF'] ) === $scriptName) {
				$this->_scriptUrl = $_SERVER ['PHP_SELF'];
			} elseif (isset ( $_SERVER ['ORIG_SCRIPT_NAME'] ) && basename ( $_SERVER ['ORIG_SCRIPT_NAME'] ) === $scriptName) {
				$this->_scriptUrl = $_SERVER ['ORIG_SCRIPT_NAME'];
			} elseif (($pos = strpos ( $_SERVER ['PHP_SELF'], '/' . $scriptName )) !== false) {
				$this->_scriptUrl = substr ( $_SERVER ['SCRIPT_NAME'], 0, $pos ) . '/' . $scriptName;
			} elseif (! empty ( $_SERVER ['DOCUMENT_ROOT'] ) && strpos ( $scriptFile, $_SERVER ['DOCUMENT_ROOT'] ) === 0) {
				$this->_scriptUrl = str_replace ( '\\', '/', str_replace ( $_SERVER ['DOCUMENT_ROOT'], '', $scriptFile ) );
			} else {
				throw new InvalidConfigException ( 'Unable to determine the entry script URL.' );
			}
		}
		return $this->_scriptUrl;
	}

	/**
	 * 设置当前执行脚本的相对路径
	 *
	 * @param string $value the relative URL for the application entry script.
	 */
	public function setScriptUrl($value) {
		$this->_scriptUrl = '/' . trim ( $value, '/' );
	}
	private $_scriptFile;

	/**
	 * 返回输入脚本文件路径。
	 *
	 * @return string the entry script file path
	 */
	public function getScriptFile() {
		return isset ( $this->_scriptFile ) ? $this->_scriptFile : $_SERVER ['SCRIPT_FILENAME'];
	}

	/**
	 * 设置输入脚本文件路径。
	 *
	 * @param string $value the entry script file path.
	 */
	public function setScriptFile($value) {
		$this->_scriptFile = $value;
	}
	private $_pathInfo;

	/**
	 * 返回当前请求的URL的路径信息。包含由客户端提供的、跟在真实脚本名称之后并且在查询语句（query string）之前的路径信息
	 *
	 * @return string part of the request URL that is after the entry script and
	 *         before the question
	 *         mark. Note, the returned path info is already URL-decoded.
	 * @throws InvalidConfigException if the path info cannot be determined due
	 *         to unexpected server
	 *         configuration
	 */
	public function getPathInfo() {
		if ($this->_pathInfo === null) {
			$this->_pathInfo = $this->resolvePathInfo ();
		}
		return $this->_pathInfo;
	}

	/**
	 * 置当前请求的路径信息。主要是提供测试目的。
	 *
	 * @param string $value the path info of the current request
	 */
	public function setPathInfo($value) {
		$this->_pathInfo = ltrim ( $value, '/' );
	}

	/**
	 * 解析当前请求的URL的路径信息
	 *
	 * @return string part of the request URL that is after the entry script and
	 *         before the question
	 *         mark. Note, the returned path info is decoded.
	 * @throws InvalidConfigException if the path info cannot be determined due
	 *         to unexpected server
	 *         configuration
	 */
	protected function resolvePathInfo() {
		$pathInfo = $this->getUrl ();
		if (($pos = strpos ( $pathInfo, '?' )) !== false) {
			$pathInfo = substr ( $pathInfo, 0, $pos );
		}
		$pathInfo = urldecode ( $pathInfo );
		// try to encode in UTF8 if not so
		// http://w3.org/International/questions/qa-forms-utf-8.html
		if (! preg_match ( '%^(?:
				[\x09\x0A\x0D\x20-\x7E]              # ASCII
				| [\xC2-\xDF][\x80-\xBF]             # non-overlong 2-byte
				| \xE0[\xA0-\xBF][\x80-\xBF]         # excluding overlongs
				| [\xE1-\xEC\xEE\xEF][\x80-\xBF]{2}  # straight 3-byte
				| \xED[\x80-\x9F][\x80-\xBF]         # excluding surrogates
				| \xF0[\x90-\xBF][\x80-\xBF]{2}      # planes 1-3
				| [\xF1-\xF3][\x80-\xBF]{3}          # planes 4-15
				| \xF4[\x80-\x8F][\x80-\xBF]{2}      # plane 16
				)*$%xs', $pathInfo )) {
			$pathInfo = utf8_encode ( $pathInfo );
		}
		$scriptUrl = $this->getScriptUrl ();
		$baseUrl = $this->getBaseUrl ();
		if (strpos ( $pathInfo, $scriptUrl ) === 0) {
			$pathInfo = substr ( $pathInfo, strlen ( $scriptUrl ) );
		} elseif ($baseUrl === '' || strpos ( $pathInfo, $baseUrl ) === 0) {
			$pathInfo = substr ( $pathInfo, strlen ( $baseUrl ) );
		} elseif (isset ( $_SERVER ['PHP_SELF'] ) && strpos ( $_SERVER ['PHP_SELF'], $scriptUrl ) === 0) {
			$pathInfo = substr ( $_SERVER ['PHP_SELF'], strlen ( $scriptUrl ) );
		} else {
			throw new InvalidConfigException ( 'Unable to determine the path info of the current request.' );
		}
		if ($pathInfo [0] === '/') {
			$pathInfo = substr ( $pathInfo, 1 );
		}
		return ( string ) $pathInfo;
	}

	/**
	 * 返回当前请求的绝对URL。
	 *
	 * @return string the currently requested absolute URL.
	 */
	public function getAbsoluteUrl() {
		return $this->getHostInfo () . $this->getUrl ();
	}
	private $_url;

	/**
	 * 返回当前请求的相对URL。
	 *
	 * @return string the currently requested relative URL. Note that the URI
	 *         returned is
	 *         URL-encoded.
	 * @throws InvalidConfigException if the URL cannot be determined due to
	 *         unusual server
	 *         configuration
	 */
	public function getUrl() {
		if ($this->_url === null) {
			$this->_url = $this->resolveRequestUri ();
		}
		return $this->_url;
	}

	/**
	 * 设置当前请求的相对URL。
	 *
	 * @param string $value the request URI to be set
	 */
	public function setUrl($value) {
		$this->_url = $value;
	}

	/**
	 * 解析当前URL请求的URI部分。
	 *
	 * @return string boolean request URI portion for the currently requested
	 *         URL. Note that the URI
	 *         returned is URL-encoded.
	 * @throws InvalidConfigException if the request URI cannot be determined
	 *         due to unusual server
	 *         configuration
	 */
	protected function resolveRequestUri() {
		if (isset ( $_SERVER ['HTTP_X_REWRITE_URL'] )) { // IIS
			$requestUri = $_SERVER ['HTTP_X_REWRITE_URL'];
		} elseif (isset ( $_SERVER ['REQUEST_URI'] )) {
			$requestUri = $_SERVER ['REQUEST_URI'];
			if ($requestUri !== '' && $requestUri [0] !== '/') {
				$requestUri = preg_replace ( '/^(http|https):\/\/[^\/]+/i', '', $requestUri );
			}
		} elseif (isset ( $_SERVER ['ORIG_PATH_INFO'] )) { // IIS 5.0 CGI
			$requestUri = $_SERVER ['ORIG_PATH_INFO'];
			if (! empty ( $_SERVER ['QUERY_STRING'] )) {
				$requestUri .= '?' . $_SERVER ['QUERY_STRING'];
			}
		} else {
			throw new InvalidConfigException ( 'Unable to determine the request URI.' );
		}
		return $requestUri;
	}

	/**
	 * 返回 part of the request URL that is after the question mark.
	 *
	 * @return string part of the request URL that is after the question mark
	 */
	public function getQueryString() {
		return isset ( $_SERVER ['QUERY_STRING'] ) ? $_SERVER ['QUERY_STRING'] : '';
	}

	/**
	 * 是否是Https请求
	 *
	 * @return boolean if the request is sent via secure channel (https)
	 */
	public function isSecureConnection() {
		return isset ( $_SERVER ['HTTPS'] ) && (strcasecmp ( $_SERVER ['HTTPS'], 'on' ) === 0 || $_SERVER ['HTTPS'] == 1) || isset ( $_SERVER ['HTTP_X_FORWARDED_PROTO'] ) && strcasecmp ( $_SERVER ['HTTP_X_FORWARDED_PROTO'], 'https' ) === 0;
	}

	/**
	 * 返回服务器名称。
	 *
	 * @return string server name
	 */
	public function getServerName() {
		return $_SERVER ['SERVER_NAME'];
	}

	/**
	 * 返回服务器的端口号。
	 *
	 * @return integer server port number
	 */
	public function getServerPort() {
		return ( int ) $_SERVER ['SERVER_PORT'];
	}

	/**
	 * 返回浏览器发送Referer请求头
	 *
	 * @return string URL referrer, null if not present
	 */
	public function getReferrer() {
		return isset ( $_SERVER ['HTTP_REFERER'] ) ? $_SERVER ['HTTP_REFERER'] : null;
	}

	/**
	 * 返回User-Agent头字段用于指定浏览器或者其他客户端程序的类型和名字
	 *
	 * @return string user agent, null if not present
	 */
	public function getUserAgent() {
		return isset ( $_SERVER ['HTTP_USER_AGENT'] ) ? $_SERVER ['HTTP_USER_AGENT'] : null;
	}
	private $_userIp;

	/**
	 * 返回用户IP地址
	 *
	 * @return string user IP address
	 */
	public function getUserIP() {
		if ($this->_userIp === null) {
			$this->_userIp = $this->resolveUserIp ();
		}
		return $this->_userIp;
	}

	/**
	 * 返回访问IP 如果获取请求IP失败,则返回0.0.0.0
	 *
	 * @return string
	 */
	protected function resolveUserIp() {
		if (isset ( $_SERVER ['HTTP_CLIENT_IP'] ) && $_SERVER ['HTTP_CLIENT_IP'] != null) {
			$userIp = $_SERVER ['HTTP_CLIENT_IP'];
		} elseif (isset ( $_SERVER [' HTTP_X_FORWARDED_FOR'] ) && $_SERVER [' HTTP_X_FORWARDED_FOR'] != null) {
			$ip = strtok ( $_SERVER [' HTTP_X_FORWARDED_FOR'], ',' );
			do {
				$ip = ip2long ( $ip );
				if (! (($ip == 0) || ($ip == 0xFFFFFFFF) || ($ip == 0x7F000001) || (($ip >= 0x0A000000) && ($ip <= 0x0AFFFFFF)) || (($ip >= 0xC0A8FFFF) && ($ip <= 0xC0A80000)) || (($ip >= 0xAC1FFFFF) && ($ip <= 0xAC100000)))) {
					$userIp = long2ip ( $ip );
					return;
				}
			} while ( ($ip = strtok ( ',' )) );
		} elseif (isset ( $_SERVER ['HTTP_PROXY_USER'] ) && $_SERVER ['HTTP_PROXY_USER'] != null) {
			$userIp = $_SERVER ['HTTP_PROXY_USER'];
		} elseif (isset ( $_SERVER ['REMOTE_ADDR'] ) && $_SERVER ['REMOTE_ADDR'] != null) {
			$userIp = $_SERVER ['REMOTE_ADDR'];
		} else {
			$userIp = "0.0.0.0";
		}
		return $userIp;
	}

	/**
	 * 返回用户主机名
	 *
	 * @return string user host name, null if cannot be determined
	 */
	public function getUserHost() {
		return isset ( $_SERVER ['REMOTE_HOST'] ) ? $_SERVER ['REMOTE_HOST'] : null;
	}
	private $_port;

	/**
	 * 返回Http端口
	 *
	 * @return integer port number for insecure requests.
	 * @see setPort()
	 */
	public function getPort() {
		if ($this->_port === null) {
			$this->_port = ! $this->getIsSecureConnection () && isset ( $_SERVER ['SERVER_PORT'] ) ? ( int ) $_SERVER ['SERVER_PORT'] : 80;
		}
		return $this->_port;
	}

	/**
	 * 设置Http端口
	 *
	 * @param integer $value port number.
	 */
	public function setPort($value) {
		if ($value != $this->_port) {
			$this->_port = ( int ) $value;
			$this->_hostInfo = null;
		}
	}
	private $_securePort;

	/**
	 * 返回Https端口
	 *
	 * @return integer port number for secure requests.
	 * @see setSecurePort()
	 */
	public function getSecurePort() {
		if ($this->_securePort === null) {
			$this->_securePort = $this->getIsSecureConnection () && isset ( $_SERVER ['SERVER_PORT'] ) ? ( int ) $_SERVER ['SERVER_PORT'] : 443;
		}
		return $this->_securePort;
	}

	/**
	 * 设置Https端口
	 *
	 * @param integer $value port number.
	 */
	public function setSecurePort($value) {
		if ($value != $this->_securePort) {
			$this->_securePort = ( int ) $value;
			$this->_hostInfo = null;
		}
	}
	private $_contentTypes;

	/**
	 * 返回最终用户可接受的内容类型。
	 *
	 * @return array the content types ordered by the preference level. The
	 *         first element represents
	 *         the most preferred content type.
	 */
	public function getAcceptableContentTypes() {
		if ($this->_contentTypes === null) {
			if (isset ( $_SERVER ['HTTP_ACCEPT'] )) {
				$this->_contentTypes = $this->parseAcceptHeader ( $_SERVER ['HTTP_ACCEPT'] );
			} else {
				$this->_contentTypes = array();
			}
		}
		return $this->_contentTypes;
	}

	/**
	 * 设置最终用户可接受的内容类型。
	 *
	 * @param array $value the content types that are acceptable by the end
	 *        user. They
	 *        should be
	 *        ordered by the preference level.
	 */
	public function setAcceptableContentTypes($value) {
		$this->_contentTypes = $value;
	}

	/**
	 * 返回请求内容类型
	 *
	 * @return string request content-type. Null is returned if this information
	 *         is not available.
	 * @link http://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html#sec14.17
	 *       HTTP 1.1 header field
	 *       definitions
	 */
	public function getContentType() {
		return isset ( $_SERVER ["CONTENT_TYPE"] ) ? $_SERVER ["CONTENT_TYPE"] : null;
	}
	private $_languages;

	/**
	 * 返回被最终用户所接受的语言。
	 *
	 * @return array the languages ordered by the preference level. The first
	 *         element represents the
	 *         most preferred language.
	 */
	public function getAcceptableLanguages() {
		if ($this->_languages === null) {
			if (isset ( $_SERVER ['HTTP_ACCEPT_LANGUAGE'] )) {
				$this->_languages = $this->parseAcceptHeader ( $_SERVER ['HTTP_ACCEPT_LANGUAGE'] );
			} else {
				$this->_languages = array();
			}
		}
		return $this->_languages;
	}

	/**
	 * 设置最终用户所接受的语言。
	 *
	 * @param array $value the languages that are acceptable by the end user.
	 *        They should
	 *        be ordered
	 *        by the preference level.
	 */
	public function setAcceptableLanguages($value) {
		$this->_languages = $value;
	}

	/**
	 * 解析 `Accept` (or `Accept-Language`) 头.
	 *
	 * @param string $header the header to be parsed
	 * @return array the accept values ordered by their preference level.
	 */
	protected function parseAcceptHeader($header) {
		$accepts = [ ];
		$n = preg_match_all ( '/\s*([\w\/\-\*]+)\s*(?:;\s*q\s*=\s*([\d\.]+))?[^,]*/', $header, $matches, PREG_SET_ORDER );
		for($i = 0; $i < $n; ++ $i) {
			if (! empty ( $matches [$i] [1] )) {
				$accepts [] = [
						$matches [$i] [1],
						isset ( $matches [$i] [2] ) ? ( float ) $matches [$i] [2] : 1,
						$i
				];
			}
		}
		usort ( $accepts, function ($a, $b) {
			if ($a [1] > $b [1]) {
				return - 1;
			} elseif ($a [1] < $b [1]) {
				return 1;
			} elseif ($a [0] === $b [0]) {
				return $a [2] > $b [2] ? 1 : - 1;
			} elseif ($a [0] === '*/*') {
				return 1;
			} elseif ($b [0] === '*/*') {
				return - 1;
			} else {
				$wa = $a [0] [strlen ( $a [0] ) - 1] === '*';
				$wb = $b [0] [strlen ( $b [0] ) - 1] === '*';
				if ($wa xor $wb) {
					return $wa ? 1 : - 1;
				} else {
					return $a [2] > $b [2] ? 1 : - 1;
				}
			}
		} );
		$result = [ ];
		foreach ( $accepts as $accept ) {
			$result [] = $accept [0];
		}
		return array_unique ( $result );
	}

	/**
	 * 返回用户首选语言。
	 *
	 * @param array $languages 应用程序支持的语言列表
	 * @return string the language that the application should use.
	 */
	public function getPreferredLanguage(array $languages = []) {
		if (empty ( $languages )) {
			return \Leaps\Config::get ( 'config.lang' );
		}
		foreach ( $this->getAcceptableLanguages () as $acceptableLanguage ) {
			$acceptableLanguage = str_replace ( '_', '-', strtolower ( $acceptableLanguage ) );
			foreach ( $languages as $language ) {
				$language = str_replace ( '_', '-', strtolower ( $language ) );
				// en-us==en-us, en==en-us, en-us==en
				if ($language === $acceptableLanguage || strpos ( $acceptableLanguage, $language . '-' ) === 0 || strpos ( $language, $acceptableLanguage . '-' ) === 0) {
					return $language;
				}
			}
		}
		return reset ( $languages );
	}

	/**
	 * 获取客户端浏览器信息
	 *
	 * @access public
	 * @return string
	 */
	public function getUserBrowser() {
		$userAgentInfo = htmlspecialchars ( $_SERVER ['HTTP_USER_AGENT'] );
		if (strpos ( $userAgentInfo, 'rv:11.0' )) {
			return 'IE11';
		} else if (strpos ( $userAgentInfo, 'MSIE 10.0' )) {
			return 'IE10';
		} else if (strpos ( $userAgentInfo, 'MSIE 9.0' )) {
			return 'IE9';
		} else if (strpos ( $userAgentInfo, 'MSIE 8.0' )) {
			return 'IE8';
		} else if (strpos ( $userAgentInfo, 'MSIE 7.0' )) {
			return 'IE7';
		} else if (strpos ( $userAgentInfo, 'MSIE 6.0' )) {
			return 'IE6';
		} else if (strpos ( $userAgentInfo, 'Firefox' )) {
			return 'Firfox';
		} else if (strpos ( $userAgentInfo, 'Chrome' )) {
			return 'Chrome';
		} else if (strpos ( $userAgentInfo, 'Opera' )) {
			return 'Opera';
		} else if (strpos ( $userAgentInfo, 'Safari' )) {
			return 'Safari';
		} else if (strpos ( $userAgentInfo, 'MicroMessenger' )) {
			return 'WeCaht';
		} else if (strpos ( $userAgentInfo, 'UCWEB' )) {
			return 'UcWeb';
		} else if (strpos ( $userAgentInfo, 'Elinks' )) {
			return 'Elinks';
		} else if (strpos ( $userAgentInfo, 'OmniWeb' )) {
			return 'OmniWeb';
		} else if (strpos ( $userAgentInfo, 'Links' )) {
			return 'Links';
		} else if (strpos ( $userAgentInfo, 'Lynx' )) {
			return 'Lynx';
		} else if (strpos ( $userAgentInfo, 'Arora' )) {
			return 'Arora';
		} else if (strpos ( $userAgentInfo, 'Epiphany' )) {
			return 'Epiphany';
		} else if (strpos ( $userAgentInfo, 'Konqueror' )) {
			return 'Konqueror';
		} else if (strpos ( $userAgentInfo, 'EudoraWeb' )) {
			return 'EudoraWeb';
		} else if (strpos ( $userAgentInfo, 'Minimo' )) {
			return 'Minimo';
		} else if (strpos ( $userAgentInfo, 'NetFront' )) {
			return 'NetFront';
		} else if (strpos ( $userAgentInfo, 'POLARIS' )) {
			return 'Polaris';
		} else if (strpos ( $userAgentInfo, 'BlackBerry' )) {
			return 'BlackBerry';
		} else if (strpos ( $userAgentInfo, 'Nokia' )) {
			return 'Nokia';
		} else {
			return 'Others';
		}
	}

	/**
	 * 获取客户端操作系统信息
	 *
	 * @access public
	 * @return string
	 */
	public function getUserOs() {
		$userAgentInfo = htmlspecialchars ( $_SERVER ['HTTP_USER_AGENT'] );
		if (strpos ( $userAgentInfo, 'Windows NT 6.2' )) {
			return 'Windows 8';
		} else if (strpos ( $userAgentInfo, 'Windows NT 6.1' )) {
			return 'Windows 7';
		} else if (strpos ( $userAgentInfo, 'Windows NT 6.0' )) {
			return 'Windows Vista';
		} else if (strpos ( $$userAgentInfo, 'Windows NT 5.2' )) {
			return 'Windows 2003';
		} else if (strpos ( $userAgentInfo, 'Windows NT 5.1' )) {
			return 'Windows XP';
		} else if (strpos ( $userAgentInfo, 'Windows NT 5.0' )) {
			return 'Windows 2000';
		} else if (strpos ( $userAgentInfo, 'Windows ME' )) {
			return 'Windows ME';
		} else if (strpos ( $userAgentInfo, 'PPC Mac OS X' )) {
			return 'OS X PPC';
		} else if (strpos ( $userAgentInfo, 'Intel Mac OS X' )) {
			return 'OS X Intel';
		} else if (strpos ( $userAgentInfo, 'Win98' )) {
			return 'Windows 98';
		} else if (strpos ( $userAgentInfo, 'Win95' )) {
			return 'Windows 95';
		} else if (strpos ( $userAgentInfo, 'WinNT4.0' )) {
			return 'Windows NT4.0';
		} else if (strpos ( $userAgentInfo, 'Mac OS X Mach-O' )) {
			return 'OS X Mach';
		} else if (strpos ( $userAgentInfo, 'Ubuntu' )) {
			return 'Ubuntu';
		} else if (strpos ( $userAgentInfo, 'Debian' )) {
			return 'Debian';
		} else if (strpos ( $userAgentInfo, 'AppleWebKit' )) {
			return 'WebKit';
		} else if (strpos ( $userAgentInfo, 'Mint/8' )) {
			return 'Mint 8';
		} else if (strpos ( $userAgentInfo, 'Minefield' )) {
			return 'Minefield Alpha';
		} else if (strpos ( $userAgentInfo, 'gentoo' )) {
			return 'Gentoo';
		} else if (strpos ( $userAgentInfo, 'Kubuntu' )) {
			return 'Kubuntu';
		} else if (strpos ( $userAgentInfo, 'Slackware/13.0' )) {
			return 'Slackware 13';
		} else if (strpos ( $userAgentInfo, 'Fedora' )) {
			return 'Fedora';
		} else if (strpos ( $userAgentInfo, 'FreeBSD' )) {
			return 'FreeBSD';
		} else if (strpos ( $userAgentInfo, 'SunOS' )) {
			return 'SunOS';
		} else if (strpos ( $userAgentInfo, 'OpenBSD' )) {
			return 'OpenBSD';
		} else if (strpos ( $userAgentInfo, 'NetBSD' )) {
			return 'NetBSD';
		} else if (strpos ( $userAgentInfo, 'DragonFly' )) {
			return 'DragonFly';
		} else if (strpos ( $userAgentInfo, 'IRIX' )) {
			return 'IRIX';
		} else if (strpos ( $userAgentInfo, 'Windows CE' )) {
			return 'Windows CE';
		} else if (strpos ( $userAgentInfo, 'PalmOS' )) {
			return 'PalmOS';
		} else if (strpos ( $userAgentInfo, 'Linux' )) {
			return 'Linux';
		} else if (strpos ( $userAgentInfo, 'DragonFly' )) {
			return 'DragonFly';
		} else if (strpos ( $userAgentInfo, 'Android' )) {
			return 'Android';
		} else if (strpos ( $userAgentInfo, 'Mac OS X' )) {
			return 'Mac OS X';
		} else if (strpos ( $userAgentInfo, 'iPhone' )) {
			return 'iPhone OS';
		} else if (strpos ( $userAgentInfo, 'Symbian OS' )) {
			return 'Symbian';
		} else if (strpos ( $userAgentInfo, 'Symbian OS' )) {
			return 'Symbian';
		} else if (strpos ( $userAgentInfo, 'SymbianOS' )) {
			return 'SymbianOS';
		} else if (strpos ( $userAgentInfo, 'webOS' )) {
			return 'webOS';
		} else if (strpos ( $userAgentInfo, 'PalmSource' )) {
			return 'PalmSource';
		} else {
			return 'Others';
		}
	}

	/**
	 * 采用stripslashes反转义特殊字符
	 *
	 * @param array|string $data 待反转义的数据
	 * @return array string
	 */
	private function _stripSlashes(&$data) {
		return is_array ( $data ) ? array_map ( [
				$this,
				'_stripSlashes'
		], $data ) : stripslashes ( $data );
	}
	private $_isConsoleRequest;

	/**
	 * 返回是否是通过命令行请求
	 *
	 * @return boolean 返回当前请求是否是通过命令行
	 */
	public function getIsConsoleRequest() {
		return $this->_isConsoleRequest !== null ? $this->_isConsoleRequest : PHP_SAPI === 'cli';
	}

	/**
	 * 设置通过命令行请求
	 *
	 * @param boolean $value 设置当前请求是通过命令行
	 */
	public function setIsConsoleRequest($value) {
		$this->_isConsoleRequest = $value;
	}
}