<?php
/**
 *
 * @author Tongle Xu <xutongle@gmail.com> 2013-11-5
 * @copyright Copyright (c) 2003-2103 tintsoft.com
 * @license http://www.tintsoft.com
 * @version $Id$
 */
defined ( 'IN_PHPCMS' ) or exit ( 'No permission resources.' );
pc_base::load_sys_class ( 'UrlRule1', '', false );
class UrlManager {

	/**
	 * 启用漂亮的URL解析
	 *
	 * @var bool
	 */
	public $enablePrettyUrl = false;

	/**
	 * 是否启用严格解析。如果启用了严格的解析,传入的请求的URL必须匹配的至少一个[[rules]]为了被视为有效请求。
	 * 否则,路径信息请求将被视为所请求的一部分路由。这个属性只在[[urlFormat]]是path。
	 *
	 * @var boolean
	 */
	public $enableStrictParsing = false;

	/**
	 * 启用URLRule缓存
	 *
	 * @var bool
	 */
	public $enableRuleCache = true;

	/**
	 * 路由规则
	 *
	 * @var array
	 */
	public $rules = array();

	/**
	 * 自定义URL后缀
	 *
	 * @var string
	 */
	public $suffix = '';

	/**
	 * 是否显示脚本名称
	 *
	 * @var boolean
	 */
	public $showScriptName = false;
	public $ruleConfig = array( 'class' => 'UrlRule1' );
	private $_baseUrl;
	private $_hostInfo;
	private $request;

	/**
	 * 构造方法 默认的实现做了两件事:
	 * - 使用给定的初始化对象配置。 - Call [[init()]].
	 * 如果在子类重写此方法,建议调用父实现的构造函数。
	 *
	 * @param array $config 名称-值对将用于初始化对象属性
	 */
	public function __construct($config = array()) {
		if (! empty ( $config )) {
			configure ( $this, $config );
		}
		$this->request = pc_base::load_sys_class ( 'Request' );
		$this->init ();
	}

	/**
	 * 初始化 UrlManager.
	 */
	public function init() {
		if (! $this->enablePrettyUrl || empty ( $this->rules )) {
			return;
		}
		/*
		if ($this->enableRuleCache) {
			$cacheKey = 'rules';
			$hash = md5 ( json_encode ( $this->rules ) );
			if (($data = getcache ( $cacheKey, 'router' )) !== false && isset ( $data [1] ) && $data [1] === $hash) {
				$this->rules = $data [0];
			} else {
				$this->rules = $this->buildRules ( $this->rules );
				setcache ( $cacheKey, [ $this->rules,$hash ], 'router' );
			}
		} else {
		*/
			$this->rules = $this->buildRules ( $this->rules );
		//}
	}

	/**
	 * 解析URL请求
	 */
	public function parseRequest() {
		$pathInfo = $this->request->getPathInfo ();
		if ($this->enablePrettyUrl && ! empty ( $pathInfo )) {
			$result = $this->_parse ( $this->request );
			if ($result !== false) {
				list ( $route, $params ) = $result;
				$_GET = array_merge ( $_GET, $params );
				$args = explode ( '/', $route );
				if (! empty ( $args ))
					$_GET ['m'] = array_shift ( $args );
				if (! empty ( $args ))
					$_GET ['c'] = array_shift ( $args );
				if (! empty ( $args ))
					$_GET ['a'] = array_shift ( $args );
			} else {
				return;
			}
		}
		return;
	}

	/**
	 * 解析路由
	 *
	 * @param unknown $request
	 * @return void unknown boolean multitype:multitype:
	 */
	protected function _parse($request) {
		$pathInfo = $request->getPathInfo ();
		/**
		 *
		 * @var UrlRule $rule
		 */
		foreach ( $this->rules as $rule ) {
			if (($result = $rule->parseRequest ( $this, $request )) !== false) {
				return $result;
			}
		}
		if ($this->enableStrictParsing) {
			return false;
		}
		$suffix = ( string ) $this->suffix;
		if ($suffix !== '' && $pathInfo !== '') {
			$n = strlen ( $this->suffix );
			if (substr ( $pathInfo, - $n ) === $this->suffix) {
				$pathInfo = substr ( $pathInfo, 0, - $n );
				if ($pathInfo === '') {
					// suffix alone is not allowed
					return false;
				}
			} else {
				// suffix doesn't match
				return false;
			}
		}
		return array( $pathInfo,array() );
	}

	/**
	 * 增加额外的URLRule
	 *
	 * 该方法调用 [[buildRules()]]解析规则并添加到现有的[[rules]].
	 *
	 * 如果没有启用漂亮的URL，该方法将什么都不做
	 *
	 * @param array $rules 规则
	 * @param boolean $append 添加到现有规则
	 */
	public function addRules($rules, $append = true) {
		if (! $this->enablePrettyUrl) {
			return;
		}
		$rules = $this->buildRules ( $rules );
		if ($append) {
			$this->rules = array_merge ( $this->rules, $rules );
		} else {
			$this->rules = array_merge ( $rules, $this->rules );
		}
	}

	/**
	 * 从Rule配置建立规则实例
	 *
	 * @param array $rules 规则
	 * @return UrlRuleInterface[] 返回规则对象实例
	 * @throws InvalidConfigException 如果Url规则不合法
	 */
	protected function buildRules($rules) {
		$compiledRules = array();
		$verbs = 'GET|HEAD|POST|PUT|PATCH|DELETE|OPTIONS';
		foreach ( $rules as $key => $rule ) {
			if (is_string ( $rule )) {
				$rule = [ 'route' => $rule ];
				if (preg_match ( "/^((?:($verbs),)*($verbs))\\s+(.*)$/", $key, $matches )) {
					$rule ['verb'] = explode ( ',', $matches [1] );
					$rule ['mode'] = UrlRule1::PARSING_ONLY;
					$key = $matches [4];
				}
				$rule ['pattern'] = $key;
			}
			if (is_array ( $rule )) {
				$rule = createObject ( array_merge ( $this->ruleConfig, $rule ) );
			}
			if (! $rule instanceof UrlRuleInterface) {
				throw new \Exception ( 'URL rule class must implement UrlRuleInterface.' );
			}
			$compiledRules [] = $rule;
		}
		return $compiledRules;
	}

	/**
	 * 创建一个相对URL。 使用 [[createAbsoluteUrl()]] 创建绝对URL
	 *
	 * @param string $route the route
	 * @param array $params the parameters (name-value pairs)
	 * @return string the created URL
	 */
	public function createUrl($params = array()) {
		$params = ( array ) $params;
		$anchor = isset ( $params ['#'] ) ? '#' . $params ['#'] : '';
		unset ( $params ['#'] );

		$route = trim ( $params [0], '/' );
		unset ( $params [0] );
		$baseUrl = $this->getBaseUrl ();
		if ($this->enablePrettyUrl && ! defined ( 'IN_ADMIN' )) {
			/**
			 *
			 * @var UrlRule $rule
			 */
			foreach ( $this->rules as $rule ) {
				if (($url = $rule->createUrl ( $this, $route, $params )) !== false) {
					if ($rule->host !== null) {
						if ($baseUrl !== '' && ($pos = strpos ( $url, '/', 8 )) !== false) {
							return substr ( $url, 0, $pos ) . $baseUrl . substr ( $url, $pos );
						} else {
							return $url . $baseUrl . $anchor;
						}
					} else {
						return "$baseUrl/{$url}{$anchor}";
					}
				}
			}

			if ($this->suffix !== null) {
				$route .= $this->suffix;
			}
			if (! empty ( $params )) {
				$route .= '?' . http_build_query ( $params );
			}
			return "$baseUrl/{$route}{$anchor}";
		} else {
			$path = explode ( '/', $route );
			$var = [ ];
			$var [$this->appParam] = isset ( $path [0] ) ? $path [0] : MODULE_NAME;
			if (isset ( $path [1] ))
				$var [$this->controllerParam] = $path [1];
			if (isset ( $path [2] ))
				$var [$this->actionParam] = $path [2];
			if ($params)
				$var = array_merge ( $var, $params );
			return $baseUrl . '?' . http_build_query ( $var ) . $anchor;
		}
	}

	/**
	 * 创建绝对URL This method prepends the URL created by [[createUrl()]] with the
	 * [[hostInfo]].
	 *
	 * @param string $route the route
	 * @param array $params the parameters (name-value pairs)
	 * @return string the created URL
	 * @see createUrl()
	 */
	public function createAbsoluteUrl($params, $schema = null) {
		$params = ( array ) $params;
		$url = $this->createUrl ( $params );
		if (strpos ( $url, '://' ) === false) {
			$url = $this->getHostInfo ( $schema ) . $url;
		}
		if ($schema !== null && ($pos = strpos ( $url, '://' )) !== false) {
			$url = $schema . substr ( $url, $pos );
		}
		return $url;
	}

	/**
	 * Returns the base URL that is used by [[createUrl()]] to prepend URLs it
	 * creates.
	 * It defaults
	 * to [[Request::scriptUrl]] if [[showScriptName]] is true or
	 * [[enablePrettyUrl]] is false;
	 * otherwise, it defaults to [[Request::baseUrl]].
	 *
	 * @return string the base URL that is used by [[createUrl()]] to prepend
	 *         URLs it creates.
	 *
	 */
	public function getBaseUrl() {
		if ($this->_baseUrl === null) {
			$this->_baseUrl = $this->showScriptName || ! $this->enablePrettyUrl ? $this->request->getScriptUrl () : $this->request->getBaseUrl ();
		}
		return $this->_baseUrl;
	}

	/**
	 * Sets the base URL that is used by [[createUrl()]] to prepend URLs it
	 * creates.
	 *
	 * @param string $value the base URL that is used by [[createUrl()]] to
	 *        prepend URLs
	 *        it creates.
	 */
	public function setBaseUrl($value) {
		$this->_baseUrl = rtrim ( $value, '/' );
	}

	/**
	 * Returns the host info that is used by [[createAbsoluteUrl()]] to prepend
	 * URLs it creates.
	 *
	 * @return string the host info (e.g. "http://www.example.com") that is used
	 *         by
	 *         [[createAbsoluteUrl()]] to prepend URLs it creates.
	 *
	 */
	public function getHostInfo() {
		if ($this->_hostInfo === null) {
			$this->_hostInfo = $this->request->getHostInfo ();
		}
		return $this->_hostInfo;
	}

	/**
	 * Sets the host info that is used by [[createAbsoluteUrl()]] to prepend
	 * URLs it creates.
	 *
	 * @param string $value the host info (e.g. "http://www.example.com") that
	 *        is used by
	 *        [[createAbsoluteUrl()]] to prepend URLs it creates.
	 */
	public function setHostInfo($value) {
		$this->_hostInfo = rtrim ( $value, '/' );
	}
}