<?php
/**
 * 使用给定的配置创建一个新的对象。 配置可以是一个字符串或一个数组。
 * 如果是字符串将被视为类名,如果是数组,必须包含'class'键指定对象类,数组的其他键值将用于初始化类的对应属性.
 * 以下是一些用法示例: ~~~ $object = \Leaps::createObject('App\Server\GoogleMap');
 * $object =
 * \Leaps::createObject([ 'class' => 'App\Server\GoogleMap', 'apiKey' =>
 * 'xyz', ]); ~~~
 * 这种方法可以用来创建任何对象,只要对象的构造函数定义如下: ~~~ public function __construct(...,
 * $config = []) { } ~~~
 *
 * @param string|array $config 配置。可以是一个字符串代表类名或一个数组表示对象配置。
 * @return mixed the created object 创建的对象
 * @throws InvalidConfigException 如果配置是无效的。
 */
function createObject($config) {
	static $reflections = [ ];
	if (is_string ( $config )) {
		$class = $config;
		$config = [ ];
	} elseif (isset ( $config ['class'] )) {
		$class = $config ['class'];
		unset ( $config ['class'] );
	} else {
		throw new \Exception ( 'Object configuration must be an array containing a "class" element.' );
	}

	$class = ltrim ( $class, '\\' );

	if (($n = func_num_args ()) > 1) {
		/**
		 *
		 * @var \ReflectionClass $reflection
		 */
		if (isset ( $reflections [$class] )) {
			$reflection = $reflections [$class];
		} else {
			$reflection = $reflections [$class] = new \ReflectionClass ( $class );
		}
		$args = func_get_args ();
		array_shift ( $args ); // remove $config
		if (! empty ( $config )) {
			$args [] = $config;
		}
		return $reflection->newInstanceArgs ( $args );
	} else {
		return empty ( $config ) ? new $class () : new $class ( $config );
	}
}

/**
 * 配置一个对象的初始属性值。
 *
 * @param object $object 对象配置
 * @param array $properties 属性初始值给定的名称-值对。
 */
function configure($object, $properties) {
	foreach ( $properties as $name => $value ) {
		$object->$name = $value;
	}
}
function router(){
		pc_base::load_sys_class('UrlManager','',false);
		$router = new UrlManager(pc_base::load_config('router'));
		return $router;
	}
function U($params, $absolute = false, $schema = null) {
	if (! $absolute) {
		return router()->createUrl ( $params );
	} else {
		return router()->createAbsoluteUrl ( $params, $schema );
	}
}
//启动URL解析
router()->parseRequest();